
option optimize_for = LITE_RUNTIME;

package TestBoolean;

message Test {

    // this should be bool, but we are using uint32
    // to be able to encode values other than 0 (false)
    // and 1 (true)
    required uint32 b = 1;

}

